@@ -45,6 +45,16 @@
 #ifndef CLONE_NEWNS
 #include <linux/sched.h>
 #endif
+#ifndef PRIO_PROCESS
+#include <sys/resource.h>
+#endif
+#ifndef MS_REC
+#define MS_REC 16384
+#endif
+#ifndef MS_PRIVATE
+#define MS_PRIVATE  (1 << 18)
+#endif
+int unshare(int flags);
 int reboot(int cmd);
 struct module;
 int init_module(const char *name, struct module *image);
@@ -63,7 +73,6 @@ static inline long sys_kexec_load(unsigned long entry,
 static char *args[512];
 static int audit_fd = EOF;
 static int policy_fd = EOF;
-static const char *bin_true_location = "/bin/true";
 static int one = 1;
 
 static const char *make_variable(const char *name, const unsigned int type)
@@ -326,6 +335,11 @@ retry:
 		 */
 		else if (strstr(cp1, ".ino"))
 			pos--;
+		/*
+		 * Ignore .minor which might change for each test.
+		 */
+		else if (strstr(cp1, ".minor"))
+			pos--;
 		cp1 = cp2;
 		if (!cp1)
 			break;
@@ -353,12 +367,6 @@ static void startup(void)
 	FILE *fp = fopen("/proc/caitsith/policy", "r");
 	policy_fd = open("/proc/caitsith/policy", O_WRONLY);
 	audit_fd = open("/proc/caitsith/audit", O_RDONLY);
-	if (access(bin_true_location, X_OK))
-		bin_true_location = "/usr/bin/true";
-	if (access(bin_true_location, X_OK)) {
-		fprintf(stderr, "***** Can't find /bin/true program.\n");
-		exit(1);
-	}
 	if (!fp || policy_fd == EOF || audit_fd == EOF) {
 		fprintf(stderr,
 			"***** Can't open /proc/caitsith/ interface.\n");
@@ -430,42 +438,37 @@ static void startup(void)
 
 static int test_execute(void)
 {
-	execlp(bin_true_location, "true", NULL);
+	execlp(BINDIR "/true", "true", NULL);
 	return errno;
 }
 
 static int test_read(void)
 {
-	const int fd = open("/dev/null", O_RDONLY);
-	const int err = errno;
-	close(fd);
-	return err;
+	open("/dev/null", O_RDONLY);
+	return errno;
 }
 
 static int test_write(void)
 {
-	const int fd = open("/dev/null", O_WRONLY);
-	const int err = errno;
-	close(fd);
-	return err;
+	open("/dev/null", O_WRONLY);
+	return errno;
 }
 
 static int test_create_no_append(void)
 {
-	const int fd = open("/tmp/caitsith.tmp/file",
-			    O_CREAT | O_RDWR | O_TRUNC, 0600);
-	const int err = errno;
-	close(fd);
+	int err;
+	open("/tmp/caitsith.tmp/file", O_CREAT | O_RDWR | O_TRUNC, 0600);
+	err = errno;
 	unlink("/tmp/caitsith.tmp/file");
 	return err;
 }
 
 static int test_create_append(void)
 {
-	const int fd = open("/tmp/caitsith.tmp/file",
-			    O_CREAT | O_RDWR | O_TRUNC | O_APPEND, 0600);
-	const int err = errno;
-	close(fd);
+	int err;
+	open("/tmp/caitsith.tmp/file", O_CREAT | O_RDWR | O_TRUNC | O_APPEND,
+	     0600);
+	err = errno;
 	unlink("/tmp/caitsith.tmp/file");
 	return err;
 }
@@ -473,21 +476,19 @@ static int test_create_append(void)
 static int test_fcntl_clear_append(void)
 {
 	const int fd = open("/dev/null", O_WRONLY | O_APPEND);
-	int err;
-	fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) & ~O_APPEND);
-	err = errno;
-	close(fd);
-	return err;
+	const int flags = fcntl(fd, F_GETFL);
+	errno = 0;
+	fcntl(fd, F_SETFL, flags & ~O_APPEND);
+	return errno;
 }
 
 static int test_fcntl_set_append(void)
 {
 	const int fd = open("/dev/null", O_WRONLY);
-	int err;
-	fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_APPEND);
-	err = errno;
-	close(fd);
-	return err;
+	const int flags = fcntl(fd, F_GETFL);
+	errno = 0;
+	fcntl(fd, F_SETFL, flags | O_APPEND);
+	return errno;
 }
 
 static int test_unlink(void)
@@ -552,7 +553,6 @@ static int test_mksock_by_bind(void)
 	errno = 0;
 	bind(fd, (struct sockaddr *) &addr, sizeof(addr));
 	err = errno;
-	close(fd);
 	unlink(addr.sun_path);
 	return err;
 }
@@ -637,11 +637,9 @@ static int test_chgrp(void)
 static int test_ioctl(void)
 {
 	const int fd = open("/dev/null", O_WRONLY);
-	int err;
+	errno = 0;
 	ioctl(fd, 0, 0);
-	err = errno;
-	close(fd);
-	return err;
+	return errno;
 }
 
 static int test_chroot(void)
@@ -652,11 +650,8 @@ static int test_chroot(void)
 
 static int test_mount(void)
 {
-	int err;
 	mount(NULL, "/tmp/caitsith.tmp", "tmpfs", 0, NULL);
-	err = errno;
-	umount("/tmp/caitsith.tmp");
-	return err;
+	return errno;
 }
 
 static int test_unmount(void)
@@ -686,7 +681,6 @@ static void inet_bind(const int fd, const unsigned short int port)
 static int test_inet_stream_bind(void)
 {
 	int fd = socket(PF_INET, SOCK_STREAM, 0);
-	int err;
 	struct sockaddr_in addr = { };
 	setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
 	addr.sin_family = AF_INET;
@@ -694,28 +688,22 @@ static int test_inet_stream_bind(void)
 	addr.sin_port = htons(10000);
 	errno = 0;
 	bind(fd, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd);
-	return err;
+	return errno;
 }
 
 static int test_inet_stream_listen(void)
 {
 	int fd = socket(PF_INET, SOCK_STREAM, 0);
-	int err;
 	inet_bind(fd, 10000);
 	errno = 0;
 	listen(fd, 5);
-	err = errno;
-	close(fd);
-	return err;
+	return errno;
 }
 
 static int test_inet_stream_connect(void)
 {
 	int fd1 = socket(PF_INET, SOCK_STREAM, 0);
 	int fd2 = socket(PF_INET, SOCK_STREAM, 0);
-	int err;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
 	inet_bind(fd1, 10000);
@@ -724,18 +712,13 @@ static int test_inet_stream_connect(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	errno = 0;
 	connect(fd2, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static int test_inet_stream_accept(void)
 {
 	int fd1 = socket(PF_INET, SOCK_STREAM, 0);
 	int fd2 = socket(PF_INET, SOCK_STREAM, 0);
-	int fd3;
-	int err;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
 	inet_bind(fd1, 10000);
@@ -744,20 +727,13 @@ static int test_inet_stream_accept(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	connect(fd2, (struct sockaddr *) &addr, sizeof(addr));
 	errno = 0;
-	fd3 = accept(fd1, (struct sockaddr *) &addr, &size);
-	if (fd3 != EOF)
-		getsockname(fd3, (struct sockaddr *) &addr, &size);
-	err = errno;
-	close(fd1);
-	close(fd2);
-	close(fd3);
-	return err;
+	accept(fd1, (struct sockaddr *) &addr, &size);
+	return errno;
 }
 
 static int test_inet_dgram_bind(void)
 {
 	int fd = socket(PF_INET, SOCK_DGRAM, 0);
-	int err;
 	struct sockaddr_in addr = { };
 	setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
 	addr.sin_family = AF_INET;
@@ -765,16 +741,13 @@ static int test_inet_dgram_bind(void)
 	addr.sin_port = htons(10000);
 	errno = 0;
 	bind(fd, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd);
-	return err;
+	return errno;
 }
 
 static int test_inet_dgram_connect(void)
 {
 	int fd1 = socket(PF_INET, SOCK_DGRAM, 0);
 	int fd2 = socket(PF_INET, SOCK_DGRAM, 0);
-	int err;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
 	inet_bind(fd1, 10000);
@@ -782,17 +755,13 @@ static int test_inet_dgram_connect(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	errno = 0;
 	connect(fd2, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static int test_inet_dgram_send(void)
 {
 	int fd1 = socket(PF_INET, SOCK_DGRAM, 0);
 	int fd2 = socket(PF_INET, SOCK_DGRAM, 0);
-	int err;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
 	inet_bind(fd1, 10000);
@@ -800,17 +769,13 @@ static int test_inet_dgram_send(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	errno = 0;
 	sendto(fd2, "", 1, 0, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static int test_inet_dgram_recv(void)
 {
 	int fd1 = socket(PF_INET, SOCK_DGRAM, 0);
 	int fd2 = socket(PF_INET, SOCK_DGRAM, 0);
-	int err;
 	char c;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
@@ -820,16 +785,12 @@ static int test_inet_dgram_recv(void)
 	sendto(fd2, "", 1, 0, (struct sockaddr *) &addr, sizeof(addr));
 	errno = 0;
 	recvfrom(fd1, &c, 1, MSG_DONTWAIT, (struct sockaddr *) &addr, &size);
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static int test_inet_raw_bind(void)
 {
 	int fd = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
-	int err;
 	struct sockaddr_in addr = { };
 	setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
 	addr.sin_family = AF_INET;
@@ -837,16 +798,13 @@ static int test_inet_raw_bind(void)
 	addr.sin_port = htons(10000);
 	errno = 0;
 	bind(fd, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd);
-	return err;
+	return errno;
 }
 
 static int test_inet_raw_connect(void)
 {
 	int fd1 = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
 	int fd2 = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
-	int err;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
 	inet_bind(fd1, 10000);
@@ -854,17 +812,13 @@ static int test_inet_raw_connect(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	errno = 0;
 	connect(fd2, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static int test_inet_raw_send(void)
 {
 	int fd1 = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
 	int fd2 = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
-	int err;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
 	static struct iphdr ip = { };
@@ -879,17 +833,13 @@ static int test_inet_raw_send(void)
 	errno = 0;
 	sendto(fd2, &ip, sizeof(ip), 0, (struct sockaddr *) &addr,
 	       sizeof(addr));
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static int test_inet_raw_recv(void)
 {
 	int fd1 = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
 	int fd2 = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
-	int err;
 	struct sockaddr_in addr = { };
 	socklen_t size = sizeof(addr);
 	static struct iphdr ip = { };
@@ -906,10 +856,7 @@ static int test_inet_raw_recv(void)
 	errno = 0;
 	recvfrom(fd1, &ip, sizeof(ip), MSG_DONTWAIT, (struct sockaddr *) &addr,
 		 &size);
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static void unix_bind(const int fd, const _Bool listener)
@@ -926,33 +873,26 @@ static void unix_bind(const int fd, const _Bool listener)
 static int test_unix_stream_bind(void)
 {
 	int fd = socket(PF_UNIX, SOCK_STREAM, 0);
-	int err;
 	struct sockaddr_un addr = { };
 	addr.sun_family = AF_UNIX;
 	errno = 0;
 	bind(fd, (struct sockaddr *) &addr, sizeof(addr.sun_family));
-	err = errno;
-	close(fd);
-	return err;
+	return errno;
 }
 
 static int test_unix_stream_listen(void)
 {
 	int fd = socket(PF_UNIX, SOCK_STREAM, 0);
-	int err;
 	unix_bind(fd, 1);
 	errno = 0;
 	listen(fd, 5);
-	err = errno;
-	close(fd);
-	return err;
+	return errno;
 }
 
 static int test_unix_stream_connect(void)
 {
 	int fd1 = socket(PF_UNIX, SOCK_STREAM, 0);
 	int fd2 = socket(PF_UNIX, SOCK_STREAM, 0);
-	int err;
 	struct sockaddr_un addr = { };
 	socklen_t size = sizeof(addr);
 	unix_bind(fd1, 1);
@@ -961,18 +901,13 @@ static int test_unix_stream_connect(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	errno = 0;
 	connect(fd2, (struct sockaddr *) &addr, sizeof(addr));
-	err = errno;
-	close(fd1);
-	close(fd2);
-	return err;
+	return errno;
 }
 
 static int test_unix_stream_accept(void)
 {
 	int fd1 = socket(PF_UNIX, SOCK_STREAM, 0);
 	int fd2 = socket(PF_UNIX, SOCK_STREAM, 0);
-	int fd3;
-	int err;
 	struct sockaddr_un addr = { };
 	socklen_t size = sizeof(addr);
 	unix_bind(fd1, 1);
@@ -981,14 +916,8 @@ static int test_unix_stream_accept(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	connect(fd2, (struct sockaddr *) &addr, sizeof(addr));
 	errno = 0;
-	fd3 = accept(fd1, (struct sockaddr *) &addr, &size);
-	if (fd3 != EOF)
-		getsockname(fd3, (struct sockaddr *) &addr, &size);
-	err = errno;
-	close(fd1);
-	close(fd2);
-	close(fd3);
-	return err;
+	accept(fd1, (struct sockaddr *) &addr, &size);
+	return errno;
 }
 
 static int test_unix_dgram_bind(void)
@@ -1108,8 +1037,6 @@ static int test_unix_seqpacket_accept(void)
 {
 	int fd1 = socket(PF_UNIX, SOCK_SEQPACKET, 0);
 	int fd2 = socket(PF_UNIX, SOCK_SEQPACKET, 0);
-	int fd3;
-	int err;
 	struct sockaddr_un addr = { };
 	socklen_t size = sizeof(addr);
 	unix_bind(fd1, 1);
@@ -1118,21 +1045,15 @@ static int test_unix_seqpacket_accept(void)
 	getsockname(fd1, (struct sockaddr *) &addr, &size);
 	connect(fd2, (struct sockaddr *) &addr, sizeof(addr));
 	errno = 0;
-	fd3 = accept(fd1, (struct sockaddr *) &addr, &size);
-	if (fd3 != EOF)
-		getsockname(fd3, (struct sockaddr *) &addr, &size);
-	err = errno;
-	close(fd1);
-	close(fd2);
-	close(fd3);
-	return err;
+	accept(fd1, (struct sockaddr *) &addr, &size);
+	return errno;
 }
 
 static int test_environ(void)
 {
 	char * const argv[2] = { "true", NULL };
 	char * const envp[2] = { "HOME=/", NULL };
-	execve(bin_true_location, argv, envp);
+	execve(BINDIR "/true", argv, envp);
 	return errno;
 }
 
@@ -1191,6 +1112,7 @@ static int test_use_reboot(void)
 		errno = 0;
 		reboot(LINUX_REBOOT_CMD_CAD_ON);
 		err = errno;
+		rewind(fp);
 		fprintf(fp, "%u\n", c);
 	} else {
 		errno = 0;
@@ -1374,6 +1296,10 @@ int main(int argc, char *argv[])
 		{ "use_new_kernel", test_use_new_kernel },
 		{ NULL, NULL }
 	};
+
+	/* Undo mount("/", MS_REC|MS_SHARED) made by systemd. */
+	mount(NULL, "/", NULL, MS_REC|MS_PRIVATE, NULL);
+
 	startup();
 	for (i = 0; testcases[i].action; i++)
 		test_action(testcases[i].action, testcases[i].func);
