@@ -31,6 +31,13 @@
 #include <linux/ip.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <sys/mount.h>
+#ifndef MS_REC
+#define MS_REC      16384
+#endif
+#ifndef MS_PRIVATE
+#define MS_PRIVATE  (1 << 18)
+#endif
 
 static FILE *fp = NULL;
 
@@ -1137,8 +1144,8 @@ static int fork_exec(char *envp[])
 	}
 	pid = fork();
 	if (pid == 0) {
-		char *argv[2] = { "/bin/true", NULL };
-		execve("/bin/true", argv, envp);
+		char *argv[2] = { BINDIR "/true", NULL };
+		execve(BINDIR "/true", argv, envp);
 		err = errno;
 		ret_ignored = write(pipe_fd[1], &err, sizeof(err));
 		_exit(0);
@@ -1189,7 +1196,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=\"/\"\n"
 		"1 deny\n";
 	set(policy);
@@ -1197,7 +1204,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=\"/\"\n"
 		"1 deny\n";
 	set(policy);
@@ -1205,7 +1212,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1213,7 +1220,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1221,7 +1228,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1229,7 +1236,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1237,7 +1244,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1245,7 +1252,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1253,7 +1260,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1261,7 +1268,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1269,7 +1276,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1277,7 +1284,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1298,7 +1305,7 @@ static int fork_exec2(char *argv[], char *envp[])
 	}
 	pid = fork();
 	if (pid == 0) {
-		execve("/bin/true", argv, envp);
+		execve(BINDIR "/true", argv, envp);
 		err = errno;
 		ret_ignored = write(pipe_fd[1], &err, sizeof(err));
 		_exit(0);
@@ -1319,7 +1326,7 @@ static void test_file_execute(void)
 	memset(argv, 0, sizeof(argv));
 	memset(envp, 0, sizeof(envp));
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argc=1\n"
 		"1 deny\n";
 	set(policy);
@@ -1327,49 +1334,49 @@ static void test_file_execute(void)
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argc!=1\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argc!=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argc=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]!=\"true\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]=\"true\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]!=\"true\"\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]=\"true\"\n"
 		"1 deny\n";
 	set(policy);
@@ -1378,7 +1385,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]!=@EXEC_ARGV0\n"
 		"1 allow\n";
 	set(policy);
@@ -1387,7 +1394,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]=@EXEC_ARGV0\n"
 		"1 allow\n";
 	set(policy);
@@ -1396,7 +1403,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]!=@EXEC_ARGV0\n"
 		"1 deny\n";
 	set(policy);
@@ -1405,7 +1412,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]=@EXEC_ARGV0\n"
 		"1 deny\n";
 	set(policy);
@@ -1413,7 +1420,7 @@ static void test_file_execute(void)
 	unset2(policy);
 
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envc=1\n"
 		"1 deny\n";
 	set(policy);
@@ -1421,117 +1428,117 @@ static void test_file_execute(void)
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envc!=1\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envc!=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envc=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]!=\"/\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]=\"/\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]!=\"/\"\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]=\"/\"\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]!=@PATH_VALUES\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset2(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]=@PATH_VALUES\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset2(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]!=@PATH_VALUES\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset2(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]=@PATH_VALUES\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset2(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]!=NULL\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]=NULL\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1773,6 +1780,9 @@ static void test_file_misc(void)
 
 int main(int argc, char *argv[])
 {
+	/* Undo mount("/", MS_REC|MS_SHARED) made by systemd. */
+	mount(NULL, "/", NULL, MS_REC|MS_PRIVATE, NULL);
+
 	fp = fopen("/proc/caitsith/policy", "w");
 	if (!fp) {
 		fprintf(stderr, " Can't open /proc/caitsith/policy\n");
