@@ -11,6 +11,13 @@
 #include <linux/ip.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <sys/mount.h>
+#ifndef MS_REC
+#define MS_REC      16384
+#endif
+#ifndef MS_PRIVATE
+#define MS_PRIVATE  (1 << 18)
+#endif
 
 static FILE *fp = NULL;
 
@@ -1225,9 +1232,12 @@ static void test_capability(void)
 
 static void detach_init(void)
 {
-	ptrace(PTRACE_DETACH, 1, NULL, NULL);
-	kill(1, SIGCONT);
-	sleep(1);
+	int i;
+	for (i = 0; i < 2; i++) {
+		ptrace(PTRACE_DETACH, 1, NULL, NULL);
+		kill(1, SIGCONT);
+		sleep(1);
+	}
 }
 
 static void test_ptrace(void)
@@ -1335,8 +1345,8 @@ static int fork_exec(char *envp[])
 	}
 	pid = fork();
 	if (pid == 0) {
-		char *argv[2] = { "/bin/true", NULL };
-		execve("/bin/true", argv, envp);
+		char *argv[2] = { BINDIR "/true", NULL };
+		execve(BINDIR "/true", argv, envp);
 		err = errno;
 		ret_ignored = write(pipe_fd[1], &err, sizeof(err));
 		_exit(0);
@@ -1387,7 +1397,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=\"/\"\n"
 		"1 deny\n";
 	set(policy);
@@ -1395,7 +1405,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=\"/\"\n"
 		"1 deny\n";
 	set(policy);
@@ -1403,7 +1413,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1411,7 +1421,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1419,7 +1429,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1427,7 +1437,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1435,7 +1445,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1443,7 +1453,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH2\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1451,7 +1461,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1459,7 +1469,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1467,7 +1477,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1475,7 +1485,7 @@ static void test_environ(void)
 	check(policy, fork_exec(envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1496,7 +1506,7 @@ static int fork_exec2(char *argv[], char *envp[])
 	}
 	pid = fork();
 	if (pid == 0) {
-		execve("/bin/true", argv, envp);
+		execve(BINDIR "/true", argv, envp);
 		err = errno;
 		ret_ignored = write(pipe_fd[1], &err, sizeof(err));
 		_exit(0);
@@ -1517,7 +1527,7 @@ static void test_file_execute(void)
 	memset(argv, 0, sizeof(argv));
 	memset(envp, 0, sizeof(envp));
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argc=1\n"
 		"1 deny\n";
 	set(policy);
@@ -1525,49 +1535,49 @@ static void test_file_execute(void)
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argc!=1\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argc!=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argc=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]!=\"true\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]=\"true\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]!=\"true\"\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]=\"true\"\n"
 		"1 deny\n";
 	set(policy);
@@ -1576,7 +1586,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]!=@EXEC_ARGV0\n"
 		"1 allow\n";
 	set(policy);
@@ -1585,7 +1595,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny argv[0]=@EXEC_ARGV0\n"
 		"1 allow\n";
 	set(policy);
@@ -1594,7 +1604,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]!=@EXEC_ARGV0\n"
 		"1 deny\n";
 	set(policy);
@@ -1603,7 +1613,7 @@ static void test_file_execute(void)
 
 	policy = "string_group EXEC_ARGV0 false\n"
 		"string_group EXEC_ARGV0 true\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow argv[0]=@EXEC_ARGV0\n"
 		"1 deny\n";
 	set(policy);
@@ -1611,7 +1621,7 @@ static void test_file_execute(void)
 	unset2(policy);
 
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envc=1\n"
 		"1 deny\n";
 	set(policy);
@@ -1619,117 +1629,117 @@ static void test_file_execute(void)
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envc!=1\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envc!=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envc=1\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]!=\"/\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]=\"/\"\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]!=\"/\"\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]=\"/\"\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]!=@PATH_VALUES\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset2(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]=@PATH_VALUES\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset2(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]!=@PATH_VALUES\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset2(policy);
 
-	policy = "string_group PATH_VALUES /bin\n"
+	policy = "string_group PATH_VALUES " BINDIR "\n"
 		"string_group PATH_VALUES /\n"
 		"string_group PATH_VALUES /sbin\n"
-		"100 acl execute path=\"/bin/true\"\n"
+		"100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]=@PATH_VALUES\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset2(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]!=NULL\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == EOF);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 deny envp[\"PATH\"]=NULL\n"
 		"1 allow\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]!=NULL\n"
 		"1 deny\n";
 	set(policy);
 	check(policy, fork_exec2(argv, envp) == 0);
 	unset(policy);
 
-	policy = "100 acl execute path=\"/bin/true\"\n"
+	policy = "100 acl execute path=\"" BINDIR "/true\"\n"
 		"0 allow envp[\"PATH\"]=NULL\n"
 		"1 deny\n";
 	set(policy);
@@ -1971,13 +1981,27 @@ static void test_file_misc(void)
 
 int main(int argc, char *argv[])
 {
+	FILE *fp2 = fopen("/proc/caitsith/policy", "r");
 	fp = fopen("/proc/caitsith/policy", "w");
-	if (!fp) {
+	if (!fp || !fp2) {
 		fprintf(stderr, " Can't open /proc/caitsith/policy\n");
 		return 1;
 	}
+	while (1) {
+		const int c = fgetc(fp2);
+		if (c == EOF)
+			break;
+		fputc(c, fp);
+		if (c == '\n')
+			fprintf(fp, "delete ");
+	}
+	fclose(fp2);
 	fprintf(fp, "quota audit[0]"
 		" allowed=1024 unmatched=1024 denied=1024\n");
+	fflush(fp);
+
+	/* Undo mount("/", MS_REC|MS_SHARED) made by systemd. */
+	mount(NULL, "/", NULL, MS_REC|MS_PRIVATE, NULL);
 
 	test_task_transition();
 	test_file_read();
