@@ -27,6 +27,13 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <time.h>
+#include <sys/mount.h>
+#ifndef MS_REC
+#define MS_REC      16384
+#endif
+#ifndef MS_PRIVATE
+#define MS_PRIVATE  (1 << 18)
+#endif
 
 enum ccs_mac_index {
 	CCS_MAC_EXECUTE,
@@ -892,6 +899,10 @@ static void check_righthand(const int mac, const int p1, const _Bool org_valid)
 
 int main(int argc, char *argv[]) {
 	int mac;
+
+	/* Undo mount("/", MS_REC|MS_SHARED) made by systemd. */
+	mount(NULL, "/", NULL, MS_REC|MS_PRIVATE, NULL);
+
 	if (argc > 1)
 		tests_start = atoi(argv[1]);
 	for (mac = 0; mac < CCS_MAX_MAC_INDEX; mac++) {
